@@ -452,7 +452,6 @@ class TestCollection(unittest.TestCase):
     def test_safe_insert(self):
         db = self.db
         db.drop_collection("test")
-        db.test.create_index("_id", ASCENDING)
 
         a = {"hello": "world"}
         db.test.insert(a)
@@ -498,7 +497,17 @@ class TestCollection(unittest.TestCase):
         self.assertRaises(OperationFailure, db.test.update,
                           {}, {"$inc": {"x": 1}}, safe=True)
 
-    # TODO test safe save?
+    def test_safe_save(self):
+        db = self.db
+        db.drop_collection("test")
+        db.test.create_index("hello", ASCENDING, unique=True)
+
+        db.test.save({"hello": "world"})
+        db.test.save({"hello": "world"})
+        self.assert_("E11000" in db.error()["err"])
+
+        self.assertRaises(OperationFailure, db.test.save, {"hello": "world"}, safe=True)
+
     def test_count(self):
         db = self.db
         db.drop_collection("test")
@@ -516,6 +525,10 @@ class TestCollection(unittest.TestCase):
                                            {"count": 0},
                                            "function (obj, prev) { "
                                            "prev.count++; }"))
+        self.assertEqual([], db.test.group([], {},
+                                           {"count": 0},
+                                           "function (obj, prev) { "
+                                           "prev.count++; }", command=True))
 
         db.test.save({"a": 2})
         db.test.save({"b": 5})
@@ -525,11 +538,41 @@ class TestCollection(unittest.TestCase):
                                           {"count": 0},
                                           "function (obj, prev) { "
                                           "prev.count++; }")[0]["count"])
+        self.assertEqual(3, db.test.group([], {},
+                                          {"count": 0},
+                                          "function (obj, prev) { "
+                                          "prev.count++; }",
+                                          command=True)[0]["count"])
         self.assertEqual(1, db.test.group([],
                                           {"a": {"$gt": 1}},
                                           {"count": 0},
                                           "function (obj, prev) { "
                                           "prev.count++; }")[0]["count"])
+        self.assertEqual(1, db.test.group([],
+                                          {"a": {"$gt": 1}},
+                                          {"count": 0},
+                                          "function (obj, prev) { "
+                                          "prev.count++; }",
+                                          command=True)[0]["count"])
+
+        db.test.save({"a": 2, "b": 3})
+
+        # NOTE maybe we can't count on this ordering being right
+        expected = [{"a": 2, "count": 2},
+                    {"a": None, "count": 1},
+                    {"a": 1, "count": 1}]
+        self.assertEqual(expected, db.test.group(["a"], {},
+                                                 {"count": 0},
+                                                 "function (obj, prev) { "
+                                                 "prev.count++; }",
+                                                 command=True))
+        self.assertEqual(expected, db.test.group(["a"], {},
+                                                 {"count": 0},
+                                                 "function (obj, prev) { "
+                                                 "prev.count++; }"))
+
+        self.assertRaises(OperationFailure, db.test.group, [], {}, {}, "5 ++ 5")
+        self.assertRaises(OperationFailure, db.test.group, [], {}, {}, "5 ++ 5", command=True)
 
     def test_large_limit(self):
         db = self.db
@@ -561,5 +604,81 @@ class TestCollection(unittest.TestCase):
 
         self.assertEqual(9, sum)
 
+    def test_rename(self):
+        db = self.db
+        db.drop_collection("test")
+        db.drop_collection("foo")
+
+        self.assertRaises(TypeError, db.test.rename, 5)
+        self.assertRaises(InvalidName, db.test.rename, "")
+        self.assertRaises(InvalidName, db.test.rename, "te$t")
+        self.assertRaises(InvalidName, db.test.rename, ".test")
+        self.assertRaises(InvalidName, db.test.rename, "test.")
+        self.assertRaises(InvalidName, db.test.rename, "tes..t")
+
+        self.assertEqual(0, db.test.count())
+        self.assertEqual(0, db.foo.count())
+
+        for i in range(10):
+            db.test.insert({"x": i})
+
+        self.assertEqual(10, db.test.count())
+
+        db.test.rename("foo")
+
+        self.assertEqual(0, db.test.count())
+        self.assertEqual(10, db.foo.count())
+
+        x = 0
+        for doc in db.foo.find():
+            self.assertEqual(x, doc["x"])
+            x += 1
+
+    # doesn't really test functionality, just that the option is set correctly
+    def test_snapshot(self):
+        db = self.db
+
+        self.assertRaises(TypeError, db.test.find, snapshot=5)
+
+        list(db.test.find(snapshot=True))
+        self.assertRaises(OperationFailure, list, db.test.find(snapshot=True).sort("foo", 1))
+
+    def test_find_one(self):
+        db = self.db
+        db.drop_collection("test")
+
+        id = db.test.save({"hello": "world", "foo": "bar"})
+
+        self.assertEqual("world", db.test.find_one()["hello"])
+        self.assertEqual(db.test.find_one(id), db.test.find_one())
+        self.assertEqual(db.test.find_one(None), db.test.find_one())
+        self.assertEqual(db.test.find_one({}), db.test.find_one())
+        self.assertEqual(db.test.find_one({"hello": "world"}), db.test.find_one())
+
+        self.assert_("hello" in db.test.find_one(fields=["hello"]))
+        self.assert_("hello" not in db.test.find_one(fields=["foo"]))
+        self.assertEqual(["_id"], db.test.find_one(fields=[]).keys())
+
+        self.assertEqual(None, db.test.find_one({"hello": "foo"}))
+        self.assertEqual(None, db.test.find_one(ObjectId()))
+
+        self.assertRaises(TypeError, db.test.find_one, 6)
+
+    def test_insert_adds_id(self):
+        doc = {"hello": "world"}
+        self.db.test.insert(doc)
+        self.assert_("_id" in doc)
+
+        docs = [{"hello": "world"}, {"hello": "world"}]
+        self.db.test.insert(docs)
+        for doc in docs:
+            self.assert_("_id" in doc)
+
+    def test_save_adds_id(self):
+        doc = {"hello": "world"}
+        self.db.test.save(doc)
+        self.assert_("_id" in doc)
+
+
 if __name__ == "__main__":
     unittest.main()
